@@ -42,9 +42,9 @@
 #include	<unistd.h>
 #endif
 
-#include        "galias.h"
 #include	"glib.h"
 #include	"gprintfint.h"
+#include        "galias.h"
 
 #ifdef G_OS_WIN32
 #include	<io.h>		/* For _read() */
@@ -74,7 +74,7 @@ struct	_GScannerKey
 
 
 /* --- variables --- */
-static GScannerConfig g_scanner_config_template =
+static const GScannerConfig g_scanner_config_template =
 {
   (
    " \t\r\n"
@@ -1763,3 +1763,6 @@ g_scanner_get_token_ll	(GScanner	*scanner,
   *token_p = token;
   *value_p = value;
 }
+
+#define __G_SCANNER_C__
+#include "galiasdef.c"
