@@ -47,6 +47,8 @@
 #include "gprintf.h"
 #include "gprintfint.h"
 
+#include "galias.h"
+
 #ifdef G_OS_WIN32
 #include <windows.h>
 #endif
@@ -334,6 +336,7 @@ g_ascii_strtod (const gchar *nptr,
   int decimal_point_len;
   const char *p, *decimal_point_pos;
   const char *end = NULL; /* Silence gcc */
+  int strtod_errno;
 
   g_return_val_if_fail (nptr != NULL, 0);
 
@@ -346,6 +349,8 @@ g_ascii_strtod (const gchar *nptr,
   g_assert (decimal_point_len != 0);
   
   decimal_point_pos = NULL;
+  end = NULL;
+
   if (decimal_point[0] != '.' || 
       decimal_point[1] != 0)
     {
@@ -368,48 +373,43 @@ g_ascii_strtod (const gchar *nptr,
 	    p++;
 	  
 	  if (*p == '.')
-	    {
-	      decimal_point_pos = p++;
+	    decimal_point_pos = p++;
 	      
-	      while (g_ascii_isxdigit (*p))
-		p++;
-	      
-	      if (*p == 'p' || *p == 'P')
-		p++;
-	      if (*p == '+' || *p == '-')
-		p++;
-	      while (g_ascii_isdigit (*p))
-		p++;
-	    }
+	  while (g_ascii_isxdigit (*p))
+	    p++;
+	  
+	  if (*p == 'p' || *p == 'P')
+	    p++;
+	  if (*p == '+' || *p == '-')
+	    p++;
+	  while (g_ascii_isdigit (*p))
+	    p++;
+
+	  end = p;
 	}
-      else
+      else if (g_ascii_isdigit (*p) || *p == '.')
 	{
 	  while (g_ascii_isdigit (*p))
 	    p++;
 	  
 	  if (*p == '.')
-	    {
-	      decimal_point_pos = p++;
-	      
-	      while (g_ascii_isdigit (*p))
-		p++;
-	      
-	      if (*p == 'e' || *p == 'E')
-		p++;
-	      if (*p == '+' || *p == '-')
-		p++;
-	      while (g_ascii_isdigit (*p))
-		p++;
-	    }
+	    decimal_point_pos = p++;
+	  
+	  while (g_ascii_isdigit (*p))
+	    p++;
+	  
+	  if (*p == 'e' || *p == 'E')
+	    p++;
+	  if (*p == '+' || *p == '-')
+	    p++;
+	  while (g_ascii_isdigit (*p))
+	    p++;
+
+	  end = p;
 	}
       /* For the other cases, we need not convert the decimal point */
-      end = p;
     }
 
-  /* Set errno to zero, so that we can distinguish zero results
-     and underflows */
-  errno = 0;
-  
   if (decimal_point_pos)
     {
       char *copy, *c;
@@ -426,7 +426,9 @@ g_ascii_strtod (const gchar *nptr,
       c += end - (decimal_point_pos + 1);
       *c = 0;
 
+      errno = 0;
       val = strtod (copy, &fail_pos);
+      strtod_errno = errno;
 
       if (fail_pos)
 	{
@@ -439,8 +441,7 @@ g_ascii_strtod (const gchar *nptr,
       g_free (copy);
 	  
     }
-  else if (decimal_point[0] != '.' ||
-	   decimal_point[1] != 0)
+  else if (end)
     {
       char *copy;
       
@@ -448,8 +449,10 @@ g_ascii_strtod (const gchar *nptr,
       memcpy (copy, nptr, end - nptr);
       *(copy + (end - (char *)nptr)) = 0;
       
+      errno = 0;
       val = strtod (copy, &fail_pos);
-      
+      strtod_errno = errno;
+
       if (fail_pos)
 	{
 	  fail_pos = (char *)nptr + (fail_pos - copy);
@@ -459,12 +462,16 @@ g_ascii_strtod (const gchar *nptr,
     }
   else
     {
+      errno = 0;
       val = strtod (nptr, &fail_pos);
+      strtod_errno = errno;
     }
 
   if (endptr)
     *endptr = fail_pos;
-  
+
+  errno = strtod_errno;
+
   return val;
 }
 
@@ -603,9 +610,12 @@ g_ascii_formatd (gchar       *buffer,
  * locale-sensitive system strtoull() function.
  *
  * If the correct value would cause overflow, %G_MAXUINT64
- * is returned, and %ERANGE is stored in %errno.
+ * is returned, and %ERANGE is stored in %errno.  If the base is
+ * outside the valid range, zero is returned, and %EINVAL is stored
+ * in %errno.  If the string conversion fails, zero is returned, and
+ * @endptr returns @nptr (if @endptr is non-%NULL).
  *
- * Return value: the #guint64 value.
+ * Return value: the #guint64 value or zero on error.
  *
  * Since: 2.2
  **/
@@ -1549,8 +1559,8 @@ g_ascii_strup (const gchar *str,
  * 
  * Return value: the string 
  *
- * Deprecated: This function is totally broken for the reasons discussed in 
- * the g_strncasecmp() docs - use g_ascii_strdown() or g_utf8_strdown() 
+ * Deprecated:2.2: This function is totally broken for the reasons discussed 
+ * in the g_strncasecmp() docs - use g_ascii_strdown() or g_utf8_strdown() 
  * instead.
  **/
 gchar*
@@ -1580,8 +1590,8 @@ g_strdown (gchar *string)
  * 
  * Return value: the string
  *
- * Deprecated: This function is totally broken for the reasons discussed in 
- * the g_strncasecmp() docs - use g_ascii_strup() or g_utf8_strup() instead.
+ * Deprecated:2.2: This function is totally broken for the reasons discussed 
+ * in the g_strncasecmp() docs - use g_ascii_strup() or g_utf8_strup() instead.
  **/
 gchar*
 g_strup (gchar *string)
@@ -1728,11 +1738,18 @@ g_ascii_xdigit_value (gchar c)
  *
  * Unlike the BSD strcasecmp() function, this only recognizes standard
  * ASCII letters and ignores the locale, treating all non-ASCII
- * characters as if they are not letters.
- * 
- * Return value: an integer less than, equal to, or greater than
- *               zero if @s1 is found, respectively, to be less than,
- *               to match, or to be greater than @s2.
+ * bytes as if they are not letters.
+ *
+ * This function should be used only on strings that are known to be
+ * in encodings where the bytes corresponding to ASCII letters always
+ * represent themselves. This includes UTF-8 and the ISO-8859-*
+ * charsets, but not for instance double-byte encodings like the
+ * Windows Codepage 932, where the trailing bytes of double-byte
+ * characters include all ASCII letters. If you compare two CP932
+ * strings using this function, you will get false matches.
+ *
+ * Return value: 0 if the strings match, a negative value if @s1 &lt; @s2, 
+ *   or a positive value if @s1 &gt; @s2.
  **/
 gint
 g_ascii_strcasecmp (const gchar *s1,
@@ -1768,10 +1785,12 @@ g_ascii_strcasecmp (const gchar *s1,
  * ASCII letters and ignores the locale, treating all non-ASCII
  * characters as if they are not letters.
  * 
- * Return value: an integer less than, equal to, or greater than zero
- *               if the first @n bytes of @s1 is found, respectively,
- *               to be less than, to match, or to be greater than the
- *               first @n bytes of @s2.
+ * The same warning as in g_ascii_strcasecmp() applies: Use this
+ * function only on strings known to be in encodings where bytes
+ * corresponding to ASCII letters always represent themselves.
+ *
+ * Return value: 0 if the strings match, a negative value if @s1 &lt; @s2, 
+ *   or a positive value if @s1 &gt; @s2.
  **/
 gint
 g_ascii_strncasecmp (const gchar *s1,
@@ -1810,8 +1829,8 @@ g_ascii_strncasecmp (const gchar *s1,
  * Return value: 0 if the strings match, a negative value if @s1 &lt; @s2, 
  *   or a positive value if @s1 &gt; @s2.
  *
- * Deprecated: See g_strncasecmp() for a discussion of why this function is 
- *   deprecated and how to replace it.
+ * Deprecated:2.2: See g_strncasecmp() for a discussion of why this function 
+ *   is deprecated and how to replace it.
  **/
 gint
 g_strcasecmp (const gchar *s1,
@@ -1858,7 +1877,7 @@ g_strcasecmp (const gchar *s1,
  * Return value: 0 if the strings match, a negative value if @s1 &lt; @s2, 
  *   or a positive value if @s1 &gt; @s2.
  *
- * Deprecated: The problem with g_strncasecmp() is that it does the 
+ * Deprecated:2.2: The problem with g_strncasecmp() is that it does the 
  * comparison by calling toupper()/tolower(). These functions are
  * locale-specific and operate on single bytes. However, it is impossible
  * to handle things correctly from an I18N standpoint by operating on
@@ -1875,7 +1894,7 @@ g_strcasecmp (const gchar *s1,
 gint
 g_strncasecmp (const gchar *s1,
 	       const gchar *s2,
-	       gsize n)     
+	       guint n)     
 {
 #ifdef HAVE_STRNCASECMP
   return strncasecmp (s1, s2, n);
@@ -1959,6 +1978,9 @@ g_strcompress (const gchar *source)
 	  p++;
 	  switch (*p)
 	    {
+	    case '\0':
+	      g_warning ("g_strcompress: trailing \\");
+	      goto out;
 	    case '0':  case '1':  case '2':  case '3':  case '4':
 	    case '5':  case '6':  case '7':
 	      *q = 0;
@@ -1995,6 +2017,7 @@ g_strcompress (const gchar *source)
 	*q++ = *p;
       p++;
     }
+out:
   *q = 0;
   
   return dest;
@@ -2696,3 +2719,31 @@ g_strip_context  (const gchar *msgid,
   
   return msgval;
 }
+
+
+/**
+ * g_strv_length:
+ * @str_array: a %NULL-terminated array of strings.
+ * 
+ * Returns the length of the given %NULL-terminated 
+ * string array @str_array.
+ * 
+ * Return value: length of @str_array.
+ *
+ * Since: 2.6
+ **/
+guint
+g_strv_length (gchar **str_array)
+{
+  guint i = 0;
+
+  g_return_val_if_fail (str_array != NULL, 0);
+
+  while (str_array[i])
+    ++i;
+
+  return i;
+}
+
+#define __G_STRFUNCS_C__
+#include "galiasdef.c"
