@@ -31,47 +31,44 @@ G_BEGIN_DECLS
  */
 #define G_TYPE_FUNDAMENTAL(type)	(g_type_fundamental (type))
 #define	G_TYPE_FUNDAMENTAL_MAX		(255 << G_TYPE_FUNDAMENTAL_SHIFT)
-#define	G_TYPE_FUNDAMENTAL_SHIFT	(2)
 
-
-/* Constant fundamental types
+/* Constant fundamental types,
+ * introduced by g_type_init().
  */
-typedef enum    /*< skip >*/
-{
-  /* standard types, introduced by g_type_init() */
-  G_TYPE_INVALID		=  0 << G_TYPE_FUNDAMENTAL_SHIFT,
-  G_TYPE_NONE			=  1 << G_TYPE_FUNDAMENTAL_SHIFT,
-  G_TYPE_INTERFACE		=  2 << G_TYPE_FUNDAMENTAL_SHIFT,
-  
-  /* GLib type IDs */
-  G_TYPE_CHAR			=  3 << G_TYPE_FUNDAMENTAL_SHIFT,
-  G_TYPE_UCHAR			=  4 << G_TYPE_FUNDAMENTAL_SHIFT,
-  G_TYPE_BOOLEAN		=  5 << G_TYPE_FUNDAMENTAL_SHIFT,
-  G_TYPE_INT			=  6 << G_TYPE_FUNDAMENTAL_SHIFT,
-  G_TYPE_UINT			=  7 << G_TYPE_FUNDAMENTAL_SHIFT,
-  G_TYPE_LONG			=  8 << G_TYPE_FUNDAMENTAL_SHIFT,
-  G_TYPE_ULONG			=  9 << G_TYPE_FUNDAMENTAL_SHIFT,
-  G_TYPE_INT64			= 10 << G_TYPE_FUNDAMENTAL_SHIFT,
-  G_TYPE_UINT64			= 11 << G_TYPE_FUNDAMENTAL_SHIFT,
-  G_TYPE_ENUM			= 12 << G_TYPE_FUNDAMENTAL_SHIFT,
-  G_TYPE_FLAGS			= 13 << G_TYPE_FUNDAMENTAL_SHIFT,
-  G_TYPE_FLOAT			= 14 << G_TYPE_FUNDAMENTAL_SHIFT,
-  G_TYPE_DOUBLE			= 15 << G_TYPE_FUNDAMENTAL_SHIFT,
-  G_TYPE_STRING			= 16 << G_TYPE_FUNDAMENTAL_SHIFT,
-  G_TYPE_POINTER		= 17 << G_TYPE_FUNDAMENTAL_SHIFT,
-  G_TYPE_BOXED			= 18 << G_TYPE_FUNDAMENTAL_SHIFT,
-  G_TYPE_PARAM			= 19 << G_TYPE_FUNDAMENTAL_SHIFT,
-  G_TYPE_OBJECT			= 20 << G_TYPE_FUNDAMENTAL_SHIFT,
-  
-  /* reserved fundamental type ids,
-   * mail gtk-devel-list@redhat.com for reservations
-   */
-  G_TYPE_RESERVED_GLIB_FIRST	= 21 << G_TYPE_FUNDAMENTAL_SHIFT,
-  G_TYPE_RESERVED_GLIB_LAST	= 31 << G_TYPE_FUNDAMENTAL_SHIFT,
-  G_TYPE_RESERVED_BSE_FIRST	= 32 << G_TYPE_FUNDAMENTAL_SHIFT,
-  G_TYPE_RESERVED_BSE_LAST	= 48 << G_TYPE_FUNDAMENTAL_SHIFT,
-  G_TYPE_RESERVED_USER_FIRST	= 49 << G_TYPE_FUNDAMENTAL_SHIFT
-} GTypeFundamentals;
+#define G_TYPE_INVALID			G_TYPE_MAKE_FUNDAMENTAL (0)
+#define G_TYPE_NONE			G_TYPE_MAKE_FUNDAMENTAL (1)
+#define G_TYPE_INTERFACE		G_TYPE_MAKE_FUNDAMENTAL (2)
+#define G_TYPE_CHAR			G_TYPE_MAKE_FUNDAMENTAL (3)
+#define G_TYPE_UCHAR			G_TYPE_MAKE_FUNDAMENTAL (4)
+#define G_TYPE_BOOLEAN			G_TYPE_MAKE_FUNDAMENTAL (5)
+#define G_TYPE_INT			G_TYPE_MAKE_FUNDAMENTAL (6)
+#define G_TYPE_UINT			G_TYPE_MAKE_FUNDAMENTAL (7)
+#define G_TYPE_LONG			G_TYPE_MAKE_FUNDAMENTAL (8)
+#define G_TYPE_ULONG			G_TYPE_MAKE_FUNDAMENTAL (9)
+#define G_TYPE_INT64			G_TYPE_MAKE_FUNDAMENTAL (10)
+#define G_TYPE_UINT64			G_TYPE_MAKE_FUNDAMENTAL (11)
+#define G_TYPE_ENUM			G_TYPE_MAKE_FUNDAMENTAL (12)
+#define G_TYPE_FLAGS			G_TYPE_MAKE_FUNDAMENTAL (13)
+#define G_TYPE_FLOAT			G_TYPE_MAKE_FUNDAMENTAL (14)
+#define G_TYPE_DOUBLE			G_TYPE_MAKE_FUNDAMENTAL (15)
+#define G_TYPE_STRING			G_TYPE_MAKE_FUNDAMENTAL (16)
+#define G_TYPE_POINTER			G_TYPE_MAKE_FUNDAMENTAL (17)
+#define G_TYPE_BOXED			G_TYPE_MAKE_FUNDAMENTAL (18)
+#define G_TYPE_PARAM			G_TYPE_MAKE_FUNDAMENTAL (19)
+#define G_TYPE_OBJECT			G_TYPE_MAKE_FUNDAMENTAL (20)
+
+
+/* Reserved fundamental type numbers to create new fundamental
+ * type IDs with G_TYPE_MAKE_FUNDAMENTAL().
+ * Send email to gtk-devel-list@redhat.com for reservations.
+ */
+#define	G_TYPE_FUNDAMENTAL_SHIFT	(2)
+#define	G_TYPE_MAKE_FUNDAMENTAL(x)	((GType) ((x) << G_TYPE_FUNDAMENTAL_SHIFT))
+#define G_TYPE_RESERVED_GLIB_FIRST	(21)
+#define G_TYPE_RESERVED_GLIB_LAST	(31)
+#define G_TYPE_RESERVED_BSE_FIRST	(32)
+#define G_TYPE_RESERVED_BSE_LAST	(48)
+#define G_TYPE_RESERVED_USER_FIRST	(49)
 
 
 /* Type Checking Macros
@@ -91,7 +88,11 @@ typedef enum    /*< skip >*/
 
 /* Typedefs
  */
+#if	GLIB_SIZEOF_LONG == GLIB_SIZEOF_SIZE_T
+typedef gulong                          GType;
+#else	/* hm, shouldn't happen? */
 typedef gsize                           GType;
+#endif
 typedef struct _GValue                  GValue;
 typedef union  _GTypeCValue             GTypeCValue;
 typedef struct _GTypePlugin             GTypePlugin;
@@ -149,6 +150,8 @@ struct _GTypeQuery
 #define G_TYPE_FROM_CLASS(g_class)                              (((GTypeClass*) (g_class))->g_type)
 #define G_TYPE_FROM_INTERFACE(g_iface)                          (((GTypeInterface*) (g_iface))->g_type)
 
+#define G_TYPE_INSTANCE_GET_PRIVATE(instance, g_type, c_type)   ((c_type*) g_type_instance_get_private ((GTypeInstance*) (instance), (g_type)))
+
 
 /* debug flags for g_type_init_with_debug_flags() */
 typedef enum	/*< skip >*/
@@ -174,12 +177,17 @@ gboolean              g_type_is_a                    (GType            type,
 						      GType            is_a_type);
 gpointer              g_type_class_ref               (GType            type);
 gpointer              g_type_class_peek              (GType            type);
+gpointer              g_type_class_peek_static       (GType            type);
 void                  g_type_class_unref             (gpointer         g_class);
 gpointer              g_type_class_peek_parent       (gpointer         g_class);
 gpointer              g_type_interface_peek          (gpointer         instance_class,
 						      GType            iface_type);
 gpointer              g_type_interface_peek_parent   (gpointer         g_iface);
 
+gpointer              g_type_default_interface_ref   (GType            g_type);
+gpointer              g_type_default_interface_peek  (GType            g_type);
+void                  g_type_default_interface_unref (gpointer         g_iface);
+
 /* g_free() the returned arrays */
 GType*                g_type_children                (GType            type,
 						      guint           *n_children);
@@ -211,6 +219,8 @@ typedef void   (*GInterfaceFinalizeFunc)     (gpointer         g_iface,
 					      gpointer         iface_data);
 typedef gboolean (*GTypeClassCacheFunc)	     (gpointer	       cache_data,
 					      GTypeClass      *g_class);
+typedef void     (*GTypeInterfaceCheckFunc)  (gpointer	       check_data,
+					      gpointer         g_iface);
 typedef enum    /*< skip >*/
 {
   G_TYPE_FLAG_CLASSED           = (1 << 0),
@@ -231,7 +241,7 @@ struct _GTypeInfo
   GBaseInitFunc          base_init;
   GBaseFinalizeFunc      base_finalize;
   
-  /* classed types, instantiated types */
+  /* interface types, classed types, instantiated types */
   GClassInitFunc         class_init;
   GClassFinalizeFunc     class_finalize;
   gconstpointer          class_data;
@@ -277,6 +287,14 @@ GType g_type_register_static		(GType			     parent_type,
 					 const gchar		    *type_name,
 					 const GTypeInfo	    *info,
 					 GTypeFlags		     flags);
+GType g_type_register_static_simple     (GType                       parent_type,
+					 const gchar                *type_name,
+					 guint                       class_size,
+					 GClassInitFunc              class_init,
+					 guint                       instance_size,
+					 GInstanceInitFunc           instance_init,
+					 GTypeFlags	             flags);
+  
 GType g_type_register_dynamic		(GType			     parent_type,
 					 const gchar		    *type_name,
 					 GTypePlugin		    *plugin,
@@ -294,21 +312,102 @@ void  g_type_add_interface_dynamic	(GType			     instance_type,
 					 GTypePlugin		    *plugin);
 void  g_type_interface_add_prerequisite (GType			     interface_type,
 					 GType			     prerequisite_type);
+GType*g_type_interface_prerequisites    (GType                       interface_type,
+					 guint                      *n_prerequisites);
+void     g_type_class_add_private       (gpointer                    g_class,
+                                         gsize                       private_size);
+gpointer g_type_instance_get_private    (GTypeInstance              *instance,
+                                         GType                       private_type);
+
+
+/* --- GType boilerplate --- */
+/* convenience macros for type implementations, which for a type GtkGadget will:
+ * - prototype: static void     gtk_gadget_class_init (GtkGadgetClass *klass);
+ * - prototype: static void     gtk_gadget_init       (GtkGadget      *self);
+ * - define:    static gpointer gtk_gadget_parent_class = NULL;
+ *   gtk_gadget_parent_class is initialized prior to calling gtk_gadget_class_init()
+ * - implement: GType           gtk_gadget_get_type (void) { ... }
+ * - support custom code in gtk_gadget_get_type() after the type is registered.
+ *
+ * macro arguments: TypeName, type_name, TYPE_PARENT, CODE
+ * example: G_DEFINE_TYPE_WITH_CODE (GtkGadget, gtk_gadget, GTK_TYPE_WIDGET,
+ *                                   g_print ("GtkGadget-id: %lu\n", g_define_type_id));
+ */
+#define G_DEFINE_TYPE(TN, t_n, T_P)			    G_DEFINE_TYPE_EXTENDED (TN, t_n, T_P, 0, {})
+#define G_DEFINE_TYPE_WITH_CODE(TN, t_n, T_P, _C_)	    _G_DEFINE_TYPE_EXTENDED_BEGIN (TN, t_n, T_P, 0) {_C_;} _G_DEFINE_TYPE_EXTENDED_END()
+#define G_DEFINE_ABSTRACT_TYPE(TN, t_n, T_P)		    G_DEFINE_TYPE_EXTENDED (TN, t_n, T_P, G_TYPE_FLAG_ABSTRACT, {})
+#define G_DEFINE_ABSTRACT_TYPE_WITH_CODE(TN, t_n, T_P, _C_) _G_DEFINE_TYPE_EXTENDED_BEGIN (TN, t_n, T_P, G_TYPE_FLAG_ABSTRACT) {_C_;} _G_DEFINE_TYPE_EXTENDED_END()
+#define G_DEFINE_TYPE_EXTENDED(TN, t_n, T_P, _f_, _C_)	    _G_DEFINE_TYPE_EXTENDED_BEGIN (TN, t_n, T_P, _f_) {_C_;} _G_DEFINE_TYPE_EXTENDED_END()
+
+/* convenience macro to ease interface addition in the CODE
+ * section of G_DEFINE_TYPE_WITH_CODE() (this macro relies on
+ * the g_define_type_id present within G_DEFINE_TYPE_WITH_CODE()).
+ * usage example:
+ * G_DEFINE_TYPE_WITH_CODE (GtkTreeStore, gtk_tree_store, G_TYPE_OBJECT,
+ *                          G_IMPLEMENT_INTERFACE (GTK_TYPE_TREE_MODEL,
+ *                                                 gtk_tree_store_tree_model_init));
+ */
+#define G_IMPLEMENT_INTERFACE(TYPE_IFACE, iface_init)       { \
+  static const GInterfaceInfo g_implement_interface_info = { \
+    (GInterfaceInitFunc) iface_init \
+  }; \
+  g_type_add_interface_static (g_define_type_id, TYPE_IFACE, &g_implement_interface_info); \
+}
+
+#define _G_DEFINE_TYPE_EXTENDED_BEGIN(TypeName, type_name, TYPE_PARENT, flags) \
+\
+static void     type_name##_init              (TypeName        *self); \
+static void     type_name##_class_init        (TypeName##Class *klass); \
+static gpointer type_name##_parent_class = NULL; \
+static void     type_name##_class_intern_init (gpointer klass) \
+{ \
+  type_name##_parent_class = g_type_class_peek_parent (klass); \
+  type_name##_class_init ((TypeName##Class*) klass); \
+} \
+\
+GType \
+type_name##_get_type (void) \
+{ \
+  static GType g_define_type_id = 0; \
+  if (G_UNLIKELY (g_define_type_id == 0)) \
+    { \
+      g_define_type_id = \
+        g_type_register_static_simple (TYPE_PARENT, \
+                                       g_intern_static_string (#TypeName), \
+                                       sizeof (TypeName##Class), \
+                                       (GClassInitFunc)type_name##_class_intern_init, \
+                                       sizeof (TypeName), \
+                                       (GInstanceInitFunc)type_name##_init, \
+                                       (GTypeFlags) flags); \
+      { /* custom code follows */
+#define _G_DEFINE_TYPE_EXTENDED_END()	\
+        /* following custom code */	\
+      }					\
+    }					\
+  return g_define_type_id;		\
+} /* closes type_name##_get_type() */
 
 
 /* --- protected (for fundamental type implementations) --- */
 GTypePlugin*	 g_type_get_plugin		(GType		     type);
 GTypePlugin*	 g_type_interface_get_plugin	(GType		     instance_type,
-						 GType               implementation_type);
+						 GType               interface_type);
 GType		 g_type_fundamental_next	(void);
 GType		 g_type_fundamental		(GType		     type_id);
 GTypeInstance*   g_type_create_instance         (GType               type);
 void             g_type_free_instance           (GTypeInstance      *instance);
+
 void		 g_type_add_class_cache_func    (gpointer	     cache_data,
 						 GTypeClassCacheFunc cache_func);
 void		 g_type_remove_class_cache_func (gpointer	     cache_data,
 						 GTypeClassCacheFunc cache_func);
 void             g_type_class_unref_uncached    (gpointer            g_class);
+
+void             g_type_add_interface_check     (gpointer	         check_data,
+						 GTypeInterfaceCheckFunc check_func);
+void             g_type_remove_interface_check  (gpointer	         check_data,
+						 GTypeInterfaceCheckFunc check_func);
+
 GTypeValueTable* g_type_value_table_peek        (GType		     type);
 
 
@@ -335,6 +434,18 @@ G_CONST_RETURN gchar* g_type_name_from_instance	(GTypeInstance	*instance);
 G_CONST_RETURN gchar* g_type_name_from_class	(GTypeClass	*g_class);
 
 
+/* --- internal functions --- */
+void    g_value_c_init          (void) G_GNUC_INTERNAL; /* sync with gvalue.c */
+void    g_value_types_init      (void) G_GNUC_INTERNAL; /* sync with gvaluetypes.c */
+void    g_enum_types_init       (void) G_GNUC_INTERNAL; /* sync with genums.c */
+void    g_param_type_init       (void) G_GNUC_INTERNAL; /* sync with gparam.c */
+void    g_boxed_type_init       (void) G_GNUC_INTERNAL; /* sync with gboxed.c */
+void    g_object_type_init      (void) G_GNUC_INTERNAL; /* sync with gobject.c */
+void    g_param_spec_types_init (void) G_GNUC_INTERNAL; /* sync with gparamspecs.c */
+void    g_value_transforms_init (void) G_GNUC_INTERNAL; /* sync with gvaluetransform.c */
+void    g_signal_init           (void) G_GNUC_INTERNAL; /* sync with gsignal.c */
+
+
 /* --- implementation bits --- */
 #ifndef G_DISABLE_CAST_CHECKS
 #  define _G_TYPE_CIC(ip, gt, ct) \
@@ -346,32 +457,40 @@ G_CONST_RETURN gchar* g_type_name_from_class	(GTypeClass	*g_class);
 #  define _G_TYPE_CCC(cp, gt, ct)       ((ct*) cp)
 #endif /* G_DISABLE_CAST_CHECKS */
 #define _G_TYPE_CHI(ip)			(g_type_check_instance ((GTypeInstance*) ip))
-#define _G_TYPE_CVH(vl, gt)             (g_type_check_value_holds ((GValue*) vl, gt))
 #define _G_TYPE_CHV(vl)			(g_type_check_value ((GValue*) vl))
 #define _G_TYPE_IGC(ip, gt, ct)         ((ct*) (((GTypeInstance*) ip)->g_class))
 #define _G_TYPE_IGI(ip, gt, ct)         ((ct*) g_type_interface_peek (((GTypeInstance*) ip)->g_class, gt))
 #ifdef	__GNUC__
-#  define _G_TYPE_CIT(ip, gt)             ({ \
+#  define _G_TYPE_CIT(ip, gt)             (G_GNUC_EXTENSION ({ \
   GTypeInstance *__inst = (GTypeInstance*) ip; GType __t = gt; gboolean __r; \
   if (__inst && __inst->g_class && __inst->g_class->g_type == __t) \
     __r = TRUE; \
   else \
     __r = g_type_check_instance_is_a (__inst, __t); \
   __r; \
-})
-#  define _G_TYPE_CCT(cp, gt)             ({ \
+}))
+#  define _G_TYPE_CCT(cp, gt)             (G_GNUC_EXTENSION ({ \
   GTypeClass *__class = (GTypeClass*) cp; GType __t = gt; gboolean __r; \
   if (__class && __class->g_type == __t) \
     __r = TRUE; \
   else \
     __r = g_type_check_class_is_a (__class, __t); \
   __r; \
-})
+}))
+#  define _G_TYPE_CVH(vl, gt)             (G_GNUC_EXTENSION ({ \
+  GValue *__val = (GValue*) vl; GType __t = gt; gboolean __r; \
+  if (__val && __val->g_type == __t) \
+    __r = TRUE; \
+  else \
+    __r = g_type_check_value_holds (__val, __t); \
+  __r; \
+}))
 #else  /* !__GNUC__ */
 #  define _G_TYPE_CIT(ip, gt)             (g_type_check_instance_is_a ((GTypeInstance*) ip, gt))
 #  define _G_TYPE_CCT(cp, gt)             (g_type_check_class_is_a ((GTypeClass*) cp, gt))
+#  define _G_TYPE_CVH(vl, gt)             (g_type_check_value_holds ((GValue*) vl, gt))
 #endif /* !__GNUC__ */
-#define	G_TYPE_FLAG_RESERVED_ID_BIT	(1 << 30)
+#define	G_TYPE_FLAG_RESERVED_ID_BIT	((GType) (1 << 0))
 extern GTypeDebugFlags			_g_type_debug_flags;
 
 G_END_DECLS
